@@ -63,6 +63,14 @@ namespace KeePass.App.Configuration
 			}
 		}
 
+		private int m_nMasterKeyTries = 3;
+		[DefaultValue(3)]
+		public int MasterKeyTries
+		{
+			get { return m_nMasterKeyTries; }
+			set { m_nMasterKeyTries = value; }
+		}
+
 		private bool m_bSecureDesktop = false;
 		[DefaultValue(false)]
 		public bool MasterKeyOnSecureDesktop
@@ -129,6 +137,14 @@ namespace KeePass.App.Configuration
 			set { m_bOnMinimize = value; }
 		}
 
+		private bool m_bOnMinimizeToTray = false;
+		[DefaultValue(false)]
+		public bool LockOnWindowMinimizeToTray
+		{
+			get { return m_bOnMinimizeToTray; }
+			set { m_bOnMinimizeToTray = value; }
+		}
+
 		private bool m_bOnSessionSwitch = false;
 		[DefaultValue(false)]
 		public bool LockOnSessionSwitch
