@@ -20,14 +20,15 @@
 using System;
 using System.Collections.Generic;
 using System.ComponentModel;
+using System.Diagnostics;
 using System.Drawing;
+using System.Globalization;
+using System.IO;
+using System.Media;
+using System.Security;
 using System.Text;
 using System.Threading;
 using System.Windows.Forms;
-using System.Globalization;
-using System.Diagnostics;
-using System.Security;
-using System.Media;
 
 using KeePass.App;
 using KeePass.App.Configuration;
@@ -385,8 +386,43 @@ namespace KeePass.Forms
 
 			m_pluginManager.UnloadAllPlugins();
 			if(AppPolicy.Current.Plugins)
-				m_pluginManager.LoadAllPlugins(UrlUtil.GetFileDirectory(
-					WinUtil.GetExecutable(), false, true));
+			{
+				string[] vExclNames = new string[] {
+					AppDefs.FileNames.Program, AppDefs.FileNames.XmlSerializers,
+					AppDefs.FileNames.NativeLib32, AppDefs.FileNames.NativeLib64,
+					AppDefs.FileNames.ShInstUtil
+				};
+
+				string strPlgRoot = UrlUtil.GetFileDirectory(
+					WinUtil.GetExecutable(), false, true);
+				m_pluginManager.LoadAllPlugins(strPlgRoot, SearchOption.TopDirectoryOnly,
+					vExclNames);
+
+				if(!NativeLib.IsUnix())
+				{
+					string strPlgSub = UrlUtil.EnsureTerminatingSeparator(strPlgRoot,
+						false) + AppDefs.PluginsDir;
+					m_pluginManager.LoadAllPlugins(strPlgSub, SearchOption.AllDirectories,
+						vExclNames);
+				}
+				else // Unix
+				{
+					try
+					{
+						DirectoryInfo diPlgRoot = new DirectoryInfo(strPlgRoot);
+						foreach(DirectoryInfo diSub in diPlgRoot.GetDirectories())
+						{
+							if(diSub == null) { Debug.Assert(false); continue; }
+
+							if(string.Equals(diSub.Name, AppDefs.PluginsDir,
+								StrUtil.CaseIgnoreCmp))
+								m_pluginManager.LoadAllPlugins(diSub.FullName,
+									SearchOption.AllDirectories, vExclNames);
+						}
+					}
+					catch(Exception) { Debug.Assert(false); }
+				}
+			}
 
 			// Delete old files *after* loading plugins (when timestamps
 			// of loaded plugins have been updated already)
@@ -454,6 +490,9 @@ namespace KeePass.Forms
 
 			m_bFormLoaded = true;
 			NotifyUserActivity(); // Initialize locking timeout
+
+			if(this.FormLoadPost != null)
+				this.FormLoadPost(this, EventArgs.Empty);
 			Program.TriggerSystem.RaiseEvent(EcasEventIDs.AppLoadPost);
 		}
 
@@ -1448,26 +1487,25 @@ namespace KeePass.Forms
 			DeleteSelectedGroup();
 		}
 
-		private void OnGroupsAfterCollapse(object sender, TreeViewEventArgs e)
+		private static void HandleGroupExpandCollapse(TreeViewEventArgs e, bool bExpand)
 		{
+			if(e == null) { Debug.Assert(false); return; }
 			TreeNode tn = e.Node;
-			if(tn == null) return;
-
-			PwGroup pg = tn.Tag as PwGroup;
+			if(tn == null) { Debug.Assert(false); return; }
+			PwGroup pg = (tn.Tag as PwGroup);
 			if(pg == null) { Debug.Assert(false); return; }
 
-			pg.IsExpanded = false;
+			pg.IsExpanded = bExpand;
+		}
+
+		private void OnGroupsAfterCollapse(object sender, TreeViewEventArgs e)
+		{
+			HandleGroupExpandCollapse(e, false);
 		}
 
 		private void OnGroupsAfterExpand(object sender, TreeViewEventArgs e)
 		{
-			TreeNode tn = e.Node;
-			if(tn == null) return;
-
-			PwGroup pg = (tn.Tag as PwGroup);
-			if(pg == null) { Debug.Assert(false); return; }
-
-			pg.IsExpanded = true;
+			HandleGroupExpandCollapse(e, true);
 		}
 
 		private void OnFileSynchronize(object sender, EventArgs e)
@@ -2650,5 +2688,38 @@ namespace KeePass.Forms
 		{
 			UpdateEntryMoveMenu(false);
 		}
+
+		private void OnGroupsExpand(object sender, EventArgs e)
+		{
+			TreeNode tn = m_tvGroups.SelectedNode;
+			if(tn == null) { Debug.Assert(false); return; }
+
+			TreeNode tnTop = m_tvGroups.TopNode;
+			m_tvGroups.BeginUpdate();
+
+			tn.ExpandAll();
+
+			if(tnTop != null) m_tvGroups.TopNode = tnTop;
+			m_tvGroups.EndUpdate();
+		}
+
+		private void OnGroupsCollapse(object sender, EventArgs e)
+		{
+			TreeNode tn = m_tvGroups.SelectedNode;
+			if(tn == null) { Debug.Assert(false); return; }
+
+			TreeNode tnTop = m_tvGroups.TopNode;
+			m_tvGroups.BeginUpdate();
+
+			if(tn.Parent != null) tn.Collapse(false);
+			else
+			{
+				foreach(TreeNode tnSub in tn.Nodes)
+					tnSub.Collapse(false);
+			}
+
+			if(tnTop != null) m_tvGroups.TopNode = tnTop;
+			m_tvGroups.EndUpdate();
+		}
 	}
 }
