@@ -692,36 +692,35 @@ namespace KeePassLib
 			MergeIn(pdSource, mm, null);
 		}
 
-		/// <summary>
-		/// Synchronize the current database with another one.
-		/// </summary>
-		/// <param name="pdSource">Input database to synchronize with. This input
-		/// database is used to update the current one, but is not modified! You
-		/// must copy the current object if you want a second instance of the
-		/// synchronized database. The input database must not be seen as valid
-		/// database any more after calling <c>Synchronize</c>.</param>
-		/// <param name="mm">Merge method.</param>
-		/// <param name="slStatus">Logger to report status messages to.
-		/// May be <c>null</c>.</param>
 		public void MergeIn(PwDatabase pdSource, PwMergeMethod mm,
 			IStatusLogger slStatus)
 		{
 			if(pdSource == null) throw new ArgumentNullException("pdSource");
 
-			PwGroup pgOrgStructure = m_pgRootGroup.CloneStructure();
-			PwGroup pgSrcStructure = pdSource.m_pgRootGroup.CloneStructure();
-
 			if(mm == PwMergeMethod.CreateNewUuids)
 			{
-				pdSource.RootGroup.CreateNewItemUuids(true, true, true);
 				pdSource.RootGroup.Uuid = new PwUuid(true);
+				pdSource.RootGroup.CreateNewItemUuids(true, true, true);
 			}
 
-			GroupHandler gh = delegate(PwGroup pg)
+			// PwGroup pgOrgStructure = m_pgRootGroup.CloneStructure();
+			// PwGroup pgSrcStructure = pdSource.RootGroup.CloneStructure();
+			// Later in case 'if(mm == PwMergeMethod.Synchronize)':
+			// PwObjectPoolEx ppOrg = PwObjectPoolEx.FromGroup(pgOrgStructure);
+			// PwObjectPoolEx ppSrc = PwObjectPoolEx.FromGroup(pgSrcStructure);
+
+			PwObjectPoolEx ppOrg = PwObjectPoolEx.FromGroup(m_pgRootGroup);
+			PwObjectPoolEx ppSrc = PwObjectPoolEx.FromGroup(pdSource.RootGroup);
+
+			GroupHandler ghSrc = delegate(PwGroup pg)
 			{
 				// if(pg == pdSource.m_pgRootGroup) return true;
 
+				// Do not use ppOrg for finding the group, because new groups
+				// might have been added (which are not in the pool, and the
+				// pool should not be modified)
 				PwGroup pgLocal = m_pgRootGroup.FindGroup(pg.Uuid, true);
+
 				if(pgLocal == null)
 				{
 					PwGroup pgSourceParent = pg.ParentGroup;
@@ -744,7 +743,10 @@ namespace KeePassLib
 					PwGroup pgNew = new PwGroup(false, false);
 					pgNew.Uuid = pg.Uuid;
 					pgNew.AssignProperties(pg, false, true);
-					pgLocalContainer.AddGroup(pgNew, true);
+
+					// pgLocalContainer.AddGroup(pgNew, true);
+					InsertObjectAtBestPos<PwGroup>(pgLocalContainer.Groups, pgNew, ppSrc);
+					pgNew.ParentGroup = pgLocalContainer;
 				}
 				else // pgLocal != null
 				{
@@ -763,9 +765,13 @@ namespace KeePassLib
 				return ((slStatus != null) ? slStatus.ContinueWork() : true);
 			};
 
-			EntryHandler eh = delegate(PwEntry pe)
+			EntryHandler ehSrc = delegate(PwEntry pe)
 			{
-				PwEntry peLocal = m_pgRootGroup.FindEntry(pe.Uuid, true);
+				// PwEntry peLocal = m_pgRootGroup.FindEntry(pe.Uuid, true);
+				PwEntry peLocal = (ppOrg.GetItemByUuid(pe.Uuid) as PwEntry);
+				Debug.Assert(object.ReferenceEquals(peLocal,
+					m_pgRootGroup.FindEntry(pe.Uuid, true)));
+
 				if(peLocal == null)
 				{
 					PwGroup pgSourceParent = pe.ParentGroup;
@@ -780,7 +786,10 @@ namespace KeePassLib
 					PwEntry peNew = new PwEntry(false, false);
 					peNew.Uuid = pe.Uuid;
 					peNew.AssignProperties(pe, false, true, true);
-					pgLocalContainer.AddEntry(peNew, true);
+
+					// pgLocalContainer.AddEntry(peNew, true);
+					InsertObjectAtBestPos<PwEntry>(pgLocalContainer.Entries, peNew, ppSrc);
+					peNew.ParentGroup = pgLocalContainer;
 				}
 				else // peLocal != null
 				{
@@ -817,8 +826,8 @@ namespace KeePassLib
 				return ((slStatus != null) ? slStatus.ContinueWork() : true);
 			};
 
-			gh(pdSource.RootGroup);
-			if(!pdSource.RootGroup.TraverseTree(TraversalMethod.PreOrder, gh, eh))
+			ghSrc(pdSource.RootGroup);
+			if(!pdSource.RootGroup.TraverseTree(TraversalMethod.PreOrder, ghSrc, ehSrc))
 				throw new InvalidOperationException();
 
 			IStatusLogger slPrevStatus = m_slStatus;
@@ -826,24 +835,25 @@ namespace KeePassLib
 
 			if(mm == PwMergeMethod.Synchronize)
 			{
-				ApplyDeletions(pdSource.m_vDeletedObjects, true);
-				ApplyDeletions(m_vDeletedObjects, false);
+				RelocateGroups(ppOrg, ppSrc);
+				RelocateEntries(ppOrg, ppSrc);
+				ReorderObjects(m_pgRootGroup, ppOrg, ppSrc);
 
-				PwObjectPool ppOrgGroups = PwObjectPool.FromGroupRecursive(
-					pgOrgStructure, false);
-				PwObjectPool ppSrcGroups = PwObjectPool.FromGroupRecursive(
-					pgSrcStructure, false);
-				PwObjectPool ppOrgEntries = PwObjectPool.FromGroupRecursive(
-					pgOrgStructure, true);
-				PwObjectPool ppSrcEntries = PwObjectPool.FromGroupRecursive(
-					pgSrcStructure, true);
+				// After all relocations and reorderings
+				MergeInLocationChanged(m_pgRootGroup, ppOrg, ppSrc);
+				ppOrg = null; // Pools are now invalid, because the location
+				ppSrc = null; // changed times have been merged in
 
-				RelocateGroups(ppOrgGroups, ppSrcGroups);
-				ReorderGroups(ppOrgGroups, ppSrcGroups);
-				RelocateEntries(ppOrgEntries, ppSrcEntries);
-				ReorderEntries(ppOrgEntries, ppSrcEntries);
+				// Delete *after* relocating, because relocating might
+				// empty some groups that are marked for deletion (and
+				// objects that weren't relocated yet might prevent the
+				// deletion)
+				Dictionary<PwUuid, PwDeletedObject> dOrgDel = CreateDeletedObjectsPool();
+				MergeInDeletionInfo(pdSource.m_vDeletedObjects, dOrgDel);
+				ApplyDeletions(m_pgRootGroup, dOrgDel);
 
-				Debug.Assert(!HasDuplicateUuids());
+				// The list and the dictionary should be kept in sync
+				Debug.Assert(m_vDeletedObjects.UCount == (uint)dOrgDel.Count);
 			}
 
 			// Must be called *after* merging groups, because group UUIDs
@@ -854,6 +864,7 @@ namespace KeePassLib
 
 			MaintainBackups();
 
+			Debug.Assert(!HasDuplicateUuids());
 			m_slStatus = slPrevStatus;
 		}
 
@@ -868,79 +879,120 @@ namespace KeePassLib
 			}
 		}
 
-		private void ApplyDeletions(PwObjectList<PwDeletedObject> listDelObjects,
-			bool bCopyDeletionInfoToLocal)
+		private Dictionary<PwUuid, PwDeletedObject> CreateDeletedObjectsPool()
 		{
-			Debug.Assert(listDelObjects != null); if(listDelObjects == null) throw new ArgumentNullException("listDelObjects");
+			Dictionary<PwUuid, PwDeletedObject> d =
+				new Dictionary<PwUuid, PwDeletedObject>();
 
-			LinkedList<PwGroup> listGroupsToDelete = new LinkedList<PwGroup>();
-			LinkedList<PwEntry> listEntriesToDelete = new LinkedList<PwEntry>();
-
-			GroupHandler gh = delegate(PwGroup pg)
+			int n = (int)m_vDeletedObjects.UCount;
+			for(int i = n - 1; i >= 0; --i)
 			{
-				if(pg == m_pgRootGroup) return true;
+				PwDeletedObject pdo = m_vDeletedObjects.GetAt((uint)i);
 
-				foreach(PwDeletedObject pdo in listDelObjects)
+				PwDeletedObject pdoEx;
+				if(d.TryGetValue(pdo.Uuid, out pdoEx))
 				{
-					if(pg.Uuid.Equals(pdo.Uuid))
-					{
-						if(TimeUtil.Compare(pg.LastModificationTime,
-							pdo.DeletionTime, true) < 0)
-							listGroupsToDelete.AddLast(pg);
-					}
+					Debug.Assert(false); // Found duplicate, which should not happen
+
+					if(pdo.DeletionTime > pdoEx.DeletionTime)
+						pdoEx.DeletionTime = pdo.DeletionTime;
+
+					m_vDeletedObjects.RemoveAt((uint)i);
 				}
+				else d[pdo.Uuid] = pdo;
+			}
 
-				return ((m_slStatus != null) ? m_slStatus.ContinueWork() : true);
-			};
+			return d;
+		}
 
-			EntryHandler eh = delegate(PwEntry pe)
+		private void MergeInDeletionInfo(PwObjectList<PwDeletedObject> lSrc,
+			Dictionary<PwUuid, PwDeletedObject> dOrgDel)
+		{
+			foreach(PwDeletedObject pdoSrc in lSrc)
 			{
-				foreach(PwDeletedObject pdo in listDelObjects)
+				PwDeletedObject pdoOrg;
+				if(dOrgDel.TryGetValue(pdoSrc.Uuid, out pdoOrg)) // Update
 				{
-					if(pe.Uuid.Equals(pdo.Uuid))
-					{
-						if(TimeUtil.Compare(pe.LastModificationTime,
-							pdo.DeletionTime, true) < 0)
-							listEntriesToDelete.AddLast(pe);
-					}
+					Debug.Assert(pdoOrg.Uuid.Equals(pdoSrc.Uuid));
+
+					if(pdoSrc.DeletionTime > pdoOrg.DeletionTime)
+						pdoOrg.DeletionTime = pdoSrc.DeletionTime;
 				}
-
-				return ((m_slStatus != null) ? m_slStatus.ContinueWork() : true);
-			};
-
-			m_pgRootGroup.TraverseTree(TraversalMethod.PreOrder, gh, eh);
-
-			foreach(PwGroup pg in listGroupsToDelete)
-				pg.ParentGroup.Groups.Remove(pg);
-			foreach(PwEntry pe in listEntriesToDelete)
-				pe.ParentGroup.Entries.Remove(pe);
-
-			if(bCopyDeletionInfoToLocal)
-			{
-				foreach(PwDeletedObject pdoNew in listDelObjects)
+				else // Add
 				{
-					bool bCopy = true;
-
-					foreach(PwDeletedObject pdoLocal in m_vDeletedObjects)
-					{
-						if(pdoNew.Uuid.Equals(pdoLocal.Uuid))
-						{
-							bCopy = false;
-
-							if(pdoNew.DeletionTime > pdoLocal.DeletionTime)
-								pdoLocal.DeletionTime = pdoNew.DeletionTime;
-
-							break;
-						}
-					}
-
-					if(bCopy) m_vDeletedObjects.Add(pdoNew);
+					m_vDeletedObjects.Add(pdoSrc);
+					dOrgDel[pdoSrc.Uuid] = pdoSrc;
 				}
 			}
 		}
 
-		private void RelocateGroups(PwObjectPool ppOrgStructure,
-			PwObjectPool ppSrcStructure)
+		private void ApplyDeletions<T>(PwObjectList<T> l, Predicate<T> fCanDelete,
+			Dictionary<PwUuid, PwDeletedObject> dOrgDel)
+			where T : class, ITimeLogger, IStructureItem, IDeepCloneable<T>
+		{
+			int n = (int)l.UCount;
+			for(int i = n - 1; i >= 0; --i)
+			{
+				if((m_slStatus != null) && !m_slStatus.ContinueWork()) break;
+
+				T t = l.GetAt((uint)i);
+
+				PwDeletedObject pdo;
+				if(dOrgDel.TryGetValue(t.Uuid, out pdo))
+				{
+					Debug.Assert(t.Uuid.Equals(pdo.Uuid));
+
+					bool bDel = (TimeUtil.Compare(t.LastModificationTime,
+						pdo.DeletionTime, true) < 0);
+					bDel &= fCanDelete(t);
+
+					if(bDel) l.RemoveAt((uint)i);
+					else
+					{
+						// Prevent future deletion attempts; this also prevents
+						// delayed deletions (emptying a group could cause a
+						// group to be deleted, if the deletion was prevented
+						// before due to the group not being empty)
+						if(!m_vDeletedObjects.Remove(pdo)) { Debug.Assert(false); }
+						if(!dOrgDel.Remove(pdo.Uuid)) { Debug.Assert(false); }
+					}
+				}
+			}
+		}
+
+		private static bool SafeCanDeleteGroup(PwGroup pg)
+		{
+			if(pg == null) { Debug.Assert(false); return false; }
+
+			if(pg.Groups.UCount > 0) return false;
+			if(pg.Entries.UCount > 0) return false;
+			return true;
+		}
+
+		private static bool SafeCanDeleteEntry(PwEntry pe)
+		{
+			if(pe == null) { Debug.Assert(false); return false; }
+
+			return true;
+		}
+
+		// Apply deletions on all objects in the specified container
+		// (but not the container itself), using post-order traversal
+		// to avoid implicit deletions;
+		// https://sourceforge.net/p/keepass/bugs/1499/
+		private void ApplyDeletions(PwGroup pgContainer,
+			Dictionary<PwUuid, PwDeletedObject> dOrgDel)
+		{
+			foreach(PwGroup pg in pgContainer.Groups) // Post-order traversal
+			{
+				ApplyDeletions(pg, dOrgDel);
+			}
+
+			ApplyDeletions<PwGroup>(pgContainer.Groups, PwDatabase.SafeCanDeleteGroup, dOrgDel);
+			ApplyDeletions<PwEntry>(pgContainer.Entries, PwDatabase.SafeCanDeleteEntry, dOrgDel);
+		}
+
+		private void RelocateGroups(PwObjectPoolEx ppOrg, PwObjectPoolEx ppSrc)
 		{
 			PwObjectList<PwGroup> vGroups = m_pgRootGroup.GetGroups(true);
 
@@ -949,10 +1001,10 @@ namespace KeePassLib
 				if((m_slStatus != null) && !m_slStatus.ContinueWork()) break;
 
 				// PwGroup pgOrg = pgOrgStructure.FindGroup(pg.Uuid, true);
-				IStructureItem ptOrg = ppOrgStructure.Get(pg.Uuid);
+				IStructureItem ptOrg = ppOrg.GetItemByUuid(pg.Uuid);
 				if(ptOrg == null) continue;
 				// PwGroup pgSrc = pgSrcStructure.FindGroup(pg.Uuid, true);
-				IStructureItem ptSrc = ppSrcStructure.Get(pg.Uuid);
+				IStructureItem ptSrc = ppSrc.GetItemByUuid(pg.Uuid);
 				if(ptSrc == null) continue;
 
 				PwGroup pgOrgParent = ptOrg.ParentGroup;
@@ -966,8 +1018,8 @@ namespace KeePassLib
 
 				if(pgOrgParent.Uuid.Equals(pgSrcParent.Uuid))
 				{
-					pg.LocationChanged = ((ptSrc.LocationChanged > ptOrg.LocationChanged) ?
-						ptSrc.LocationChanged : ptOrg.LocationChanged);
+					// pg.LocationChanged = ((ptSrc.LocationChanged > ptOrg.LocationChanged) ?
+					//	ptSrc.LocationChanged : ptOrg.LocationChanged);
 					continue;
 				}
 
@@ -979,8 +1031,12 @@ namespace KeePassLib
 					if(pgLocal.IsContainedIn(pg)) continue;
 
 					pg.ParentGroup.Groups.Remove(pg);
-					pgLocal.AddGroup(pg, true);
-					pg.LocationChanged = ptSrc.LocationChanged;
+
+					// pgLocal.AddGroup(pg, true);
+					InsertObjectAtBestPos<PwGroup>(pgLocal.Groups, pg, ppSrc);
+					pg.ParentGroup = pgLocal;
+
+					// pg.LocationChanged = ptSrc.LocationChanged;
 				}
 				else
 				{
@@ -992,8 +1048,7 @@ namespace KeePassLib
 			Debug.Assert(m_pgRootGroup.GetGroups(true).UCount == vGroups.UCount);
 		}
 
-		private void RelocateEntries(PwObjectPool ppOrgStructure,
-			PwObjectPool ppSrcStructure)
+		private void RelocateEntries(PwObjectPoolEx ppOrg, PwObjectPoolEx ppSrc)
 		{
 			PwObjectList<PwEntry> vEntries = m_pgRootGroup.GetEntries(true);
 
@@ -1002,18 +1057,18 @@ namespace KeePassLib
 				if((m_slStatus != null) && !m_slStatus.ContinueWork()) break;
 
 				// PwEntry peOrg = pgOrgStructure.FindEntry(pe.Uuid, true);
-				IStructureItem ptOrg = ppOrgStructure.Get(pe.Uuid);
+				IStructureItem ptOrg = ppOrg.GetItemByUuid(pe.Uuid);
 				if(ptOrg == null) continue;
 				// PwEntry peSrc = pgSrcStructure.FindEntry(pe.Uuid, true);
-				IStructureItem ptSrc = ppSrcStructure.Get(pe.Uuid);
+				IStructureItem ptSrc = ppSrc.GetItemByUuid(pe.Uuid);
 				if(ptSrc == null) continue;
 
 				PwGroup pgOrg = ptOrg.ParentGroup;
 				PwGroup pgSrc = ptSrc.ParentGroup;
 				if(pgOrg.Uuid.Equals(pgSrc.Uuid))
 				{
-					pe.LocationChanged = ((ptSrc.LocationChanged > ptOrg.LocationChanged) ?
-						ptSrc.LocationChanged : ptOrg.LocationChanged);
+					// pe.LocationChanged = ((ptSrc.LocationChanged > ptOrg.LocationChanged) ?
+					//	ptSrc.LocationChanged : ptOrg.LocationChanged);
 					continue;
 				}
 
@@ -1023,8 +1078,12 @@ namespace KeePassLib
 					if(pgLocal == null) { Debug.Assert(false); continue; }
 
 					pe.ParentGroup.Entries.Remove(pe);
-					pgLocal.AddEntry(pe, true);
-					pe.LocationChanged = ptSrc.LocationChanged;
+
+					// pgLocal.AddEntry(pe, true);
+					InsertObjectAtBestPos<PwEntry>(pgLocal.Entries, pe, ppSrc);
+					pe.ParentGroup = pgLocal;
+
+					// pe.LocationChanged = ptSrc.LocationChanged;
 				}
 				else
 				{
@@ -1036,255 +1095,295 @@ namespace KeePassLib
 			Debug.Assert(m_pgRootGroup.GetEntries(true).UCount == vEntries.UCount);
 		}
 
-		private void ReorderGroups(PwObjectPool ppOrgStructure,
-			PwObjectPool ppSrcStructure)
+		private void ReorderObjects(PwGroup pg, PwObjectPoolEx ppOrg,
+			PwObjectPoolEx ppSrc)
 		{
-			GroupHandler gh = delegate(PwGroup pg)
-			{
-				ReorderObjectList<PwGroup>(pg.Groups, ppOrgStructure,
-					ppSrcStructure, false);
-				return true;
-			};
+			ReorderObjectList<PwGroup>(pg.Groups, ppOrg, ppSrc);
+			ReorderObjectList<PwEntry>(pg.Entries, ppOrg, ppSrc);
 
-			ReorderObjectList<PwGroup>(m_pgRootGroup.Groups, ppOrgStructure,
-				ppSrcStructure, false);
-			m_pgRootGroup.TraverseTree(TraversalMethod.PreOrder, gh, null);
+			foreach(PwGroup pgSub in pg.Groups)
+			{
+				ReorderObjects(pgSub, ppOrg, ppSrc);
+			}
 		}
 
-		private void ReorderEntries(PwObjectPool ppOrgStructure,
-			PwObjectPool ppSrcStructure)
+		private void ReorderObjectList<T>(PwObjectList<T> lItems,
+			PwObjectPoolEx ppOrg, PwObjectPoolEx ppSrc)
+			where T : class, ITimeLogger, IStructureItem, IDeepCloneable<T>
 		{
-			GroupHandler gh = delegate(PwGroup pg)
-			{
-				ReorderObjectList<PwEntry>(pg.Entries, ppOrgStructure,
-					ppSrcStructure, true);
-				return true;
-			};
-
-			ReorderObjectList<PwEntry>(m_pgRootGroup.Entries, ppOrgStructure,
-				ppSrcStructure, true);
-			m_pgRootGroup.TraverseTree(TraversalMethod.PreOrder, gh, null);
-		}
-
-		private void ReorderObjectList<T>(PwObjectList<T> vItems,
-			PwObjectPool ppOrgStructure, PwObjectPool ppSrcStructure, bool bEntries)
-			where T : class, IStructureItem, IDeepCloneable<T>
-		{
-			if(!ObjectListRequiresReorder<T>(vItems, ppOrgStructure, ppSrcStructure,
-				bEntries)) return;
+			List<PwObjectBlock<T>> lBlocks = PartitionConsec(lItems, ppOrg, ppSrc);
+			if(lBlocks.Count <= 1) return;
 
 #if DEBUG
-			PwObjectList<T> vOrgListItems = vItems.CloneShallow();
+			PwObjectList<T> lOrgItems = lItems.CloneShallow();
 #endif
 
-			Queue<KeyValuePair<uint, uint>> qToDo = new Queue<KeyValuePair<uint, uint>>();
-			qToDo.Enqueue(new KeyValuePair<uint, uint>(0, vItems.UCount - 1));
+			Queue<KeyValuePair<int, int>> qToDo = new Queue<KeyValuePair<int, int>>();
+			qToDo.Enqueue(new KeyValuePair<int, int>(0, lBlocks.Count - 1));
 
 			while(qToDo.Count > 0)
 			{
 				if((m_slStatus != null) && !m_slStatus.ContinueWork()) break;
 
-				KeyValuePair<uint, uint> kvp = qToDo.Dequeue();
-				if(kvp.Value <= kvp.Key) { Debug.Assert(false); continue; }
+				KeyValuePair<int, int> kvp = qToDo.Dequeue();
+				if(kvp.Key >= kvp.Value) { Debug.Assert(false); continue; }
 
-				Queue<PwUuid> qRelBefore = new Queue<PwUuid>();
-				Queue<PwUuid> qRelAfter = new Queue<PwUuid>();
-				uint uPivot = FindLocationChangedPivot<T>(vItems, kvp, ppOrgStructure,
-					ppSrcStructure, qRelBefore, qRelAfter, bEntries);
-				T ptPivot = vItems.GetAt(uPivot);
+				PwObjectPoolEx pPool;
+				int iPivot = FindLocationChangedPivot(lBlocks, kvp, out pPool);
+				PwObjectBlock<T> bPivot = lBlocks[iPivot];
 
-				List<T> vToSort = vItems.GetRange(kvp.Key, kvp.Value);
-				Queue<T> qBefore = new Queue<T>();
-				Queue<T> qAfter = new Queue<T>();
+				T tPivotPrimary = bPivot.PrimaryItem;
+				if(tPivotPrimary == null) { Debug.Assert(false); continue; }
+				ulong idPivot = pPool.GetIdByUuid(tPivotPrimary.Uuid);
+				if(idPivot == 0) { Debug.Assert(false); continue; }
+
+				Queue<PwObjectBlock<T>> qBefore = new Queue<PwObjectBlock<T>>();
+				Queue<PwObjectBlock<T>> qAfter = new Queue<PwObjectBlock<T>>();
 				bool bBefore = true;
 
-				foreach(T pt in vToSort)
+				for(int i = kvp.Key; i <= kvp.Value; ++i)
 				{
-					if(pt == ptPivot) { bBefore = false; continue; }
+					if(i == iPivot) { bBefore = false; continue; }
 
-					bool bAdded = false;
-					foreach(PwUuid puBefore in qRelBefore)
+					PwObjectBlock<T> b = lBlocks[i];
+					Debug.Assert(b.LocationChanged <= bPivot.LocationChanged);
+
+					T t = b.PrimaryItem;
+					if(t != null)
 					{
-						if(puBefore.Equals(pt.Uuid))
+						ulong idBPri = pPool.GetIdByUuid(t.Uuid);
+						if(idBPri > 0)
 						{
-							qBefore.Enqueue(pt);
-							bAdded = true;
-							break;
+							if(idBPri < idPivot) qBefore.Enqueue(b);
+							else qAfter.Enqueue(b);
+
+							continue;
 						}
 					}
-					if(bAdded) continue;
+					else { Debug.Assert(false); }
 
-					foreach(PwUuid puAfter in qRelAfter)
-					{
-						if(puAfter.Equals(pt.Uuid))
-						{
-							qAfter.Enqueue(pt);
-							bAdded = true;
-							break;
-						}
-					}
-					if(bAdded) continue;
-
-					if(bBefore) qBefore.Enqueue(pt);
-					else qAfter.Enqueue(pt);
-				}
-				Debug.Assert(bBefore == false);
-
-				uint uPos = kvp.Key;
-				while(qBefore.Count > 0) vItems.SetAt(uPos++, qBefore.Dequeue());
-				vItems.SetAt(uPos++, ptPivot);
-				while(qAfter.Count > 0) vItems.SetAt(uPos++, qAfter.Dequeue());
-				Debug.Assert(uPos == (kvp.Value + 1));
-
-				int iNewPivot = vItems.IndexOf(ptPivot);
-				if((iNewPivot < (int)kvp.Key) || (iNewPivot > (int)kvp.Value))
-				{
-					Debug.Assert(false);
-					continue;
+					if(bBefore) qBefore.Enqueue(b);
+					else qAfter.Enqueue(b);
 				}
 
-				if((iNewPivot - 1) > (int)kvp.Key)
-					qToDo.Enqueue(new KeyValuePair<uint, uint>(kvp.Key,
-						(uint)(iNewPivot - 1)));
+				int j = kvp.Key;
+				while(qBefore.Count > 0) { lBlocks[j] = qBefore.Dequeue(); ++j; }
+				int iNewPivot = j;
+				lBlocks[j] = bPivot;
+				++j;
+				while(qAfter.Count > 0) { lBlocks[j] = qAfter.Dequeue(); ++j; }
+				Debug.Assert(j == (kvp.Value + 1));
 
-				if((iNewPivot + 1) < (int)kvp.Value)
-					qToDo.Enqueue(new KeyValuePair<uint, uint>((uint)(iNewPivot + 1),
-						kvp.Value));
+				if((iNewPivot - 1) > kvp.Key)
+					qToDo.Enqueue(new KeyValuePair<int, int>(kvp.Key, iNewPivot - 1));
+				if((iNewPivot + 1) < kvp.Value)
+					qToDo.Enqueue(new KeyValuePair<int, int>(iNewPivot + 1, kvp.Value));
 			}
 
-#if DEBUG
-			foreach(T ptItem in vOrgListItems)
+			uint u = 0;
+			foreach(PwObjectBlock<T> b in lBlocks)
 			{
-				Debug.Assert(vItems.IndexOf(ptItem) >= 0);
+				foreach(T t in b)
+				{
+					lItems.SetAt(u, t);
+					++u;
+				}
+			}
+			Debug.Assert(u == lItems.UCount);
+
+#if DEBUG
+			Debug.Assert(u == lOrgItems.UCount);
+			foreach(T ptItem in lOrgItems)
+			{
+				Debug.Assert(lItems.IndexOf(ptItem) >= 0);
 			}
 #endif
 		}
 
-		private static uint FindLocationChangedPivot<T>(PwObjectList<T> vItems,
-			KeyValuePair<uint, uint> kvpRange, PwObjectPool ppOrgStructure,
-			PwObjectPool ppSrcStructure, Queue<PwUuid> qBefore, Queue<PwUuid> qAfter,
-			bool bEntries)
-			where T : class, IStructureItem, IDeepCloneable<T>
+		private static List<PwObjectBlock<T>> PartitionConsec<T>(PwObjectList<T> lItems,
+			PwObjectPoolEx ppOrg, PwObjectPoolEx ppSrc)
+			where T : class, ITimeLogger, IStructureItem, IDeepCloneable<T>
 		{
-			uint uPosMax = kvpRange.Key;
+			List<PwObjectBlock<T>> lBlocks = new List<PwObjectBlock<T>>();
+
+			Dictionary<PwUuid, bool> dItemUuids = new Dictionary<PwUuid, bool>();
+			foreach(T t in lItems) { dItemUuids[t.Uuid] = true; }
+
+			uint n = lItems.UCount;
+			for(uint u = 0; u < n; ++u)
+			{
+				T t = lItems.GetAt(u);
+
+				PwObjectBlock<T> b = new PwObjectBlock<T>();
+
+				DateTime dtLoc;
+				PwObjectPoolEx pPool = GetBestPool(t, ppOrg, ppSrc, out dtLoc);
+				b.Add(t, dtLoc, pPool);
+
+				lBlocks.Add(b);
+
+				ulong idOrg = ppOrg.GetIdByUuid(t.Uuid);
+				ulong idSrc = ppSrc.GetIdByUuid(t.Uuid);
+				if((idOrg == 0) || (idSrc == 0)) continue;
+
+				for(uint x = u + 1; x < n; ++x)
+				{
+					T tNext = lItems.GetAt(x);
+
+					ulong idOrgNext = idOrg + 1;
+					while(true)
+					{
+						IStructureItem ptOrg = ppOrg.GetItemById(idOrgNext);
+						if(ptOrg == null) { idOrgNext = 0; break; }
+						if(ptOrg.Uuid.Equals(tNext.Uuid)) break; // Found it
+						if(dItemUuids.ContainsKey(ptOrg.Uuid)) { idOrgNext = 0; break; }
+						++idOrgNext;
+					}
+					if(idOrgNext == 0) break;
+
+					ulong idSrcNext = idSrc + 1;
+					while(true)
+					{
+						IStructureItem ptSrc = ppSrc.GetItemById(idSrcNext);
+						if(ptSrc == null) { idSrcNext = 0; break; }
+						if(ptSrc.Uuid.Equals(tNext.Uuid)) break; // Found it
+						if(dItemUuids.ContainsKey(ptSrc.Uuid)) { idSrcNext = 0; break; }
+						++idSrcNext;
+					}
+					if(idSrcNext == 0) break;
+
+					pPool = GetBestPool(tNext, ppOrg, ppSrc, out dtLoc);
+					b.Add(tNext, dtLoc, pPool);
+
+					++u;
+					idOrg = idOrgNext;
+					idSrc = idSrcNext;
+				}
+			}
+
+			return lBlocks;
+		}
+
+		private static PwObjectPoolEx GetBestPool<T>(T t, PwObjectPoolEx ppOrg,
+			PwObjectPoolEx ppSrc, out DateTime dtLoc)
+			where T : class, ITimeLogger, IStructureItem, IDeepCloneable<T>
+		{
+			PwObjectPoolEx p = null;
+			dtLoc = DateTime.MinValue;
+
+			IStructureItem ptOrg = ppOrg.GetItemByUuid(t.Uuid);
+			if(ptOrg != null)
+			{
+				dtLoc = ptOrg.LocationChanged;
+				p = ppOrg;
+			}
+
+			IStructureItem ptSrc = ppSrc.GetItemByUuid(t.Uuid);
+			if((ptSrc != null) && (ptSrc.LocationChanged > dtLoc))
+			{
+				dtLoc = ptSrc.LocationChanged;
+				p = ppSrc;
+			}
+
+			Debug.Assert(p != null);
+			return p;
+		}
+
+		private static int FindLocationChangedPivot<T>(List<PwObjectBlock<T>> lBlocks,
+			KeyValuePair<int, int> kvpRange, out PwObjectPoolEx pPool)
+			where T : class, ITimeLogger, IStructureItem, IDeepCloneable<T>
+		{
+			pPool = null;
+
+			int iPosMax = kvpRange.Key;
 			DateTime dtMax = DateTime.MinValue;
-			List<IStructureItem> vNeighborSrc = null;
 
-			for(uint u = kvpRange.Key; u <= kvpRange.Value; ++u)
+			for(int i = kvpRange.Key; i <= kvpRange.Value; ++i)
 			{
-				T pt = vItems.GetAt(u);
-
-				// IStructureItem ptOrg = pgOrgStructure.FindObject(pt.Uuid, true, bEntries);
-				IStructureItem ptOrg = ppOrgStructure.Get(pt.Uuid);
-				if((ptOrg != null) && (ptOrg.LocationChanged > dtMax))
+				PwObjectBlock<T> b = lBlocks[i];
+				if(b.LocationChanged > dtMax)
 				{
-					uPosMax = u;
-					dtMax = ptOrg.LocationChanged; // No 'continue'
-
-					PwGroup pgParent = ptOrg.ParentGroup;
-					if(pgParent != null)
-						vNeighborSrc = pgParent.GetObjects(false, bEntries);
-					else
-					{
-						Debug.Assert(false); // Org root should be excluded
-						vNeighborSrc = new List<IStructureItem>();
-						vNeighborSrc.Add(ptOrg);
-					}
-				}
-
-				// IStructureItem ptSrc = pgSrcStructure.FindObject(pt.Uuid, true, bEntries);
-				IStructureItem ptSrc = ppSrcStructure.Get(pt.Uuid);
-				if((ptSrc != null) && (ptSrc.LocationChanged > dtMax))
-				{
-					uPosMax = u;
-					dtMax = ptSrc.LocationChanged; // No 'continue'
-
-					PwGroup pgParent = ptSrc.ParentGroup;
-					if(pgParent != null)
-						vNeighborSrc = pgParent.GetObjects(false, bEntries);
-					else
-					{
-						// pgParent may be null (for the source root group)
-						vNeighborSrc = new List<IStructureItem>();
-						vNeighborSrc.Add(ptSrc);
-					}
+					iPosMax = i;
+					dtMax = b.LocationChanged;
+					pPool = b.PoolAssoc;
 				}
 			}
 
-			GetNeighborItems(vNeighborSrc, vItems.GetAt(uPosMax).Uuid, qBefore, qAfter);
-			return uPosMax;
+			return iPosMax;
 		}
 
-		private static void GetNeighborItems(List<IStructureItem> vItems,
-			PwUuid pwPivot, Queue<PwUuid> qBefore, Queue<PwUuid> qAfter)
+		private static void MergeInLocationChanged(PwGroup pg,
+			PwObjectPoolEx ppOrg, PwObjectPoolEx ppSrc)
 		{
-			qBefore.Clear();
-			qAfter.Clear();
-
-			// Checks after clearing the queues
-			if(vItems == null) { Debug.Assert(false); return; } // No throw
-
-			bool bBefore = true;
-			for(int i = 0; i < vItems.Count; ++i)
+			GroupHandler gh = delegate(PwGroup pgSub)
 			{
-				PwUuid pw = vItems[i].Uuid;
+				DateTime dt;
+				if(GetBestPool<PwGroup>(pgSub, ppOrg, ppSrc, out dt) != null)
+					pgSub.LocationChanged = dt;
+				else { Debug.Assert(false); }
+				return true;
+			};
 
-				if(pw.Equals(pwPivot)) bBefore = false;
-				else if(bBefore) qBefore.Enqueue(pw);
-				else qAfter.Enqueue(pw);
-			}
-			Debug.Assert(bBefore == false);
+			EntryHandler eh = delegate(PwEntry pe)
+			{
+				DateTime dt;
+				if(GetBestPool<PwEntry>(pe, ppOrg, ppSrc, out dt) != null)
+					pe.LocationChanged = dt;
+				else { Debug.Assert(false); }
+				return true;
+			};
+
+			gh(pg);
+			pg.TraverseTree(TraversalMethod.PreOrder, gh, eh);
 		}
 
-		/// <summary>
-		/// Method to check whether a reordering is required. This fast test
-		/// allows to skip the reordering routine, resulting in a large
-		/// performance increase.
-		/// </summary>
-		private bool ObjectListRequiresReorder<T>(PwObjectList<T> vItems,
-			PwObjectPool ppOrgStructure, PwObjectPool ppSrcStructure, bool bEntries)
-			where T : class, IStructureItem, IDeepCloneable<T>
+		private static void InsertObjectAtBestPos<T>(PwObjectList<T> lItems,
+			T tNew, PwObjectPoolEx ppSrc)
+			where T : class, ITimeLogger, IStructureItem, IDeepCloneable<T>
 		{
-			Debug.Assert(ppOrgStructure.ContainsOnlyType(bEntries ? typeof(PwEntry) : typeof(PwGroup)));
-			Debug.Assert(ppSrcStructure.ContainsOnlyType(bEntries ? typeof(PwEntry) : typeof(PwGroup)));
-			if(vItems.UCount <= 1) return false;
+			if(tNew == null) { Debug.Assert(false); return; }
 
-			if((m_slStatus != null) && !m_slStatus.ContinueWork()) return false;
+			ulong idSrc = ppSrc.GetIdByUuid(tNew.Uuid);
+			if(idSrc == 0) { Debug.Assert(false); lItems.Add(tNew); return; }
 
-			T ptFirst = vItems.GetAt(0);
-			// IStructureItem ptOrg = pgOrgStructure.FindObject(ptFirst.Uuid, true, bEntries);
-			IStructureItem ptOrg = ppOrgStructure.Get(ptFirst.Uuid);
-			if(ptOrg == null) return true;
-			// IStructureItem ptSrc = pgSrcStructure.FindObject(ptFirst.Uuid, true, bEntries);
-			IStructureItem ptSrc = ppSrcStructure.Get(ptFirst.Uuid);
-			if(ptSrc == null) return true;
+			const uint uIdOffset = 2;
+			Dictionary<PwUuid, uint> dOrg = new Dictionary<PwUuid, uint>();
+			for(uint u = 0; u < lItems.UCount; ++u)
+				dOrg[lItems.GetAt(u).Uuid] = uIdOffset + u;
 
-			if(ptFirst.ParentGroup == null) { Debug.Assert(false); return true; }
-			PwGroup pgOrgParent = ptOrg.ParentGroup;
-			if(pgOrgParent == null) return true; // Root might be in tree
-			PwGroup pgSrcParent = ptSrc.ParentGroup;
-			if(pgSrcParent == null) return true; // Root might be in tree
-
-			if(!ptFirst.ParentGroup.Uuid.Equals(pgOrgParent.Uuid)) return true;
-			if(!pgOrgParent.Uuid.Equals(pgSrcParent.Uuid)) return true;
-
-			List<IStructureItem> lOrg = pgOrgParent.GetObjects(false, bEntries);
-			List<IStructureItem> lSrc = pgSrcParent.GetObjects(false, bEntries);
-			if(vItems.UCount != (uint)lOrg.Count) return true;
-			if(lOrg.Count != lSrc.Count) return true;
-
-			for(uint u = 0; u < vItems.UCount; ++u)
+			ulong idSrcNext = idSrc + 1;
+			uint idOrgNext = 0;
+			while(true)
 			{
-				IStructureItem pt = vItems.GetAt(u);
-				Debug.Assert(pt.ParentGroup == ptFirst.ParentGroup);
-
-				if(!pt.Uuid.Equals(lOrg[(int)u].Uuid)) return true;
-				if(!pt.Uuid.Equals(lSrc[(int)u].Uuid)) return true;
-				if(pt.LocationChanged != lOrg[(int)u].LocationChanged) return true;
-				if(pt.LocationChanged != lSrc[(int)u].LocationChanged) return true;
+				IStructureItem pNext = ppSrc.GetItemById(idSrcNext);
+				if(pNext == null) break;
+				if(dOrg.TryGetValue(pNext.Uuid, out idOrgNext)) break;
+				++idSrcNext;
 			}
 
-			return false;
+			if(idOrgNext != 0)
+			{
+				lItems.Insert(idOrgNext - uIdOffset, tNew);
+				return;
+			}
+
+			ulong idSrcPrev = idSrc - 1;
+			uint idOrgPrev = 0;
+			while(true)
+			{
+				IStructureItem pPrev = ppSrc.GetItemById(idSrcPrev);
+				if(pPrev == null) break;
+				if(dOrg.TryGetValue(pPrev.Uuid, out idOrgPrev)) break;
+				--idSrcPrev;
+			}
+
+			if(idOrgPrev != 0)
+			{
+				lItems.Insert(idOrgPrev + 1 - uIdOffset, tNew);
+				return;
+			}
+
+			lItems.Add(tNew);
 		}
 
 		private void MergeInDbProperties(PwDatabase pdSource, PwMergeMethod mm)
